package org.rcisoft.business.opmTopic.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.opmArticle.dao.OpmArticleRepository;
import org.rcisoft.business.opmArticle.entity.OpmArticle;
import org.rcisoft.business.opmTopic.dao.OpmTopicRepository;
import org.rcisoft.business.opmTopic.entity.OpmTopic;
import org.rcisoft.business.opmTopic.service.OpmTopicService;
import org.rcisoft.business.sysScheduledTaskLog.entity.SysScheduledTaskLog;
import org.rcisoft.business.sysScheduledTaskLog.service.SysScheduledTaskLogService;
import org.rcisoft.core.exception.CyServiceException;
import org.rcisoft.core.model.CyPageInfo;
import org.rcisoft.core.model.CyPersistModel;
import org.rcisoft.core.util.CyUserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

/**
 * @author cy
 * @date 2024年3月25日 下午1:42:40
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class OpmTopicServiceImpl extends ServiceImpl<OpmTopicRepository, OpmTopic> implements OpmTopicService {

    @Autowired
    private OpmArticleRepository opmArticleRepository;
    @Autowired
    private SysScheduledTaskLogService sysScheduledTaskLogServiceImpl;

    /**
     * 保存 话题
     *
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(OpmTopic opmTopic) {
        //增加操作
        opmTopic.setDelFlag("0");
        opmTopic.setExamStatus("0");
        //当图片字段没有时增加默认1
/*        if (ObjectUtil.isNull(cmsNotice.getPictureId())) {
            cmsNotice.setPictureId(global.getDEFAULT_NEWS_LOCATION());
        }*/
        OpmTopic check = baseMapper.checkWeight(opmTopic.getWeight());
        if (ObjectUtil.isNotNull(check)) {
            throw new CyServiceException(500, "权重已存在");
        } else {
            int line = baseMapper.insert(opmTopic);
            log.debug(CyUserUtil.getAuthenUsername() + "新增了ID为" +
                    opmTopic.getBusinessId() + "的话题信息");
            return new CyPersistModel(line);
        }
    }

    /**
     * 删除 话题
     *
     * @param opmTopic
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(OpmTopic opmTopic) {
        int line = baseMapper.deleteOpmTopic(opmTopic.getBusinessId());
        log.debug(CyUserUtil.getAuthenUsername() + "删除了ID为" +
                opmTopic.getBusinessId() + "的话题信息");
        return new CyPersistModel(line);
    }

    /**
     * 修改 话题
     *
     * @param opmTopic
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED, isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(OpmTopic opmTopic) {
        //修改文本中的#image
        OpmTopic check = baseMapper.checkWeight(opmTopic.getWeight());
        if (ObjectUtil.isNotNull(check) && !check.getBusinessId().equals(opmTopic.getBusinessId())) {
            throw new CyServiceException(500, "权重已存在");
        } else {
            int line = baseMapper.updateById(opmTopic);
            log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ opmTopic.getBusinessId()+"的话题管理信息");
            return new CyPersistModel(line);
        }
    }

    /**
     * 根据id查询 话题
     *
     * @param id
     * @return
     */
    @Override
    public OpmTopic findById(Integer id) {
        OpmTopic opmTopic = baseMapper.selectById(id);
        return opmTopic;
    }

    /**
     * 分页查询 话题
     *
     * @param opmTopic
     * @return
     */
    @Override
    public IPage<OpmTopic> findAllByPagination(CyPageInfo<OpmTopic> paginationUtility,
                                               OpmTopic opmTopic) {
        IPage<OpmTopic> opmTopicIPage = baseMapper.queryOpmTopicPaged(paginationUtility, opmTopic);
        return opmTopicIPage;

    }

    /**
     * 查询list 话题
     *
     * @param opmTopic
     * @return
     */
    @Override
    public List<OpmTopic> findAll(OpmTopic opmTopic,String type) {
        if (type.equals("app")){
            return baseMapper.appQueryOpmTopic(opmTopic);
        } else {
            return baseMapper.queryOpmTopic(opmTopic);
        }
    }

    /**
     * 导出话题
     *
     * @return
     */
    @Override
    public List<OpmTopic> export(OpmTopic opmTopic) {
        List<OpmTopic> opmTopicList = baseMapper.queryOpmTopic(opmTopic);
        return opmTopicList;
    }

    @Override
    public CyPersistModel updateStatus(OpmTopic opmTopic) {
        int line = baseMapper.updateStatus(opmTopic);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + opmTopic.getBusinessId() + "的话题信息状态");
        return new CyPersistModel(line);
    }

    @Override
    public CyPersistModel updateExamStatus(OpmTopic opmTopic) {
        int line = baseMapper.updateExamStatus(opmTopic);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + opmTopic.getBusinessId() + "的话题信息状态");
        return new CyPersistModel(line);
    }

    @Override
    public CyPersistModel updateNoExamStatus(OpmTopic opmTopic) {
        int line = baseMapper.updateNoExamStatus(opmTopic);
        log.debug(CyUserUtil.getAuthenUsername() + "修改了ID为" + opmTopic.getBusinessId() + "的话题信息状态");
        return new CyPersistModel(line);
    }


    /**
     * 定时更新话题表的动态数
     */
    public void opmTopicSchedule() {
       try {
           //查询所有话题
           List<OpmTopic> allTopicList = baseMapper.selectAllTopic();
           allTopicList.forEach(topic -> {
               Integer articleCount = baseMapper.selectArticleCountByTopicId(topic.getBusinessId());
               topic.setArticleCount(articleCount);
               baseMapper.updateArticleCountById(topic);
           });
           SysScheduledTaskLog sysScheduledTaskLog = new SysScheduledTaskLog();
           sysScheduledTaskLog.setTaskName("定时更新话题表动态数定时任务");
           sysScheduledTaskLog.setTaskState("成功");
           sysScheduledTaskLog.setTaskTime("每2小时执行一次");
           sysScheduledTaskLog.setCreateDate(new Date());
           sysScheduledTaskLogServiceImpl.persist(sysScheduledTaskLog);
       }  catch (Exception e) {
           SysScheduledTaskLog sysScheduledTaskLog = new SysScheduledTaskLog();
           sysScheduledTaskLog.setTaskName("定时更新话题表动态数定时任务");
           sysScheduledTaskLog.setTaskState("失败");
           sysScheduledTaskLog.setTaskTime("每2小时执行一次");
           sysScheduledTaskLog.setCreateDate(new Date());
           sysScheduledTaskLogServiceImpl.persist(sysScheduledTaskLog);
       }

    }
}
